-- EnrageTracker v1.9 by Naw
-- Track warrior enrage effects with a customizable enrage bar


-- Create the ace3 addon
local EnrageTracker = LibStub("AceAddon-3.0"):NewAddon("EnrageTracker", "AceConsole-3.0", "AceEvent-3.0")

-- Create the enraged bar
local EnragedFrame = CreateBar("Interface\\Addons\\EnrageTracker\\Textures\\smooth.tga", 100, 15)
EnragedFrame:SetMovable(true)
EnragedFrame:SetResizable(true)
EnragedFrame:SetClampedToScreen(true)
EnragedFrame:SetDontSavePosition(true)
EnragedFrame:SetMinResize(100, 10)
EnragedFrame:SetMaxResize(300, 50)
EnragedFrame:SetScript("OnMouseDown", function() EnragedFrame:StartMoving() end)
EnragedFrame:SetScript("OnMouseUp", function() EnragedFrame:StopMovingOrSizing(); EnrageTracker:SavePosition() end)

-- Register the bar textures
local SharedMedia = LibStub("LibSharedMedia-3.0")
SharedMedia:Register("statusbar", "BantoBar", "Interface\\Addons\\EnrageTracker\\Textures\\banto.tga")
SharedMedia:Register("statusbar", "Charcoal", "Interface\\Addons\\EnrageTracker\\Textures\\charcoal.tga")
SharedMedia:Register("statusbar", "Glaze", "Interface\\Addons\\EnrageTracker\\Textures\\glaze.tga")
SharedMedia:Register("statusbar", "Otravi", "Interface\\Addons\\EnrageTracker\\Textures\\otravi.tga")
SharedMedia:Register("statusbar", "Perl", "Interface\\Addons\\EnrageTracker\\Textures\\perl.tga")
SharedMedia:Register("statusbar", "Smooth", "Interface\\Addons\\EnrageTracker\\Textures\\smooth.tga")
SharedMedia:Register("statusbar", "Striped", "Interface\\Addons\\EnrageTracker\\Textures\\striped.tga")

-- List of enrage effects to track
local enrages = { "Enrage", "Death Wish", "Berserker Rage", "Unholy Frenzy" }


--------------------------------------------------------------------------------
-- Options table and defaults table
--------------------------------------------------------------------------------


-- Options table
local options = {
	type = "group",
	name = "EnrageTracker",
	handler = EnrageTracker,
	args = {
		enable = {
			type = "toggle",
			name = "Enable",
			desc = "Enable or disable the addon",
			order = 10,
			get = function(info) return EnrageTracker.db.profile.enable end,
			set = function(info, value)
				EnrageTracker.db.profile.enable = value
				EnrageTracker:ToggleAddon()
				if info.uiType == "cmd" then
					LibStub("AceConfigRegistry-3.0"):NotifyChange("EnrageTracker")
					if value then
						print("|cff00ff00Enrage Tracker:|r Enabled")
					else
						print("|cff00ff00Enrage Tracker:|r Disabled")
					end
				end
			end,
		},
		
		fury = { 
			type = "toggle",
			name = "Fury only",
			desc = "Only track enrages while specced as fury",
			order = 20,
			cmdHidden = true,
			get = function(info) return EnrageTracker.db.profile.fury end,
			set = function(info, value) 
				EnrageTracker.db.profile.fury = value
				EnrageTracker:ToggleAddon()
			end,
		},

		sizespacer = { type = "description", name = "\n", order = 29, },
		sizeheader = { type = "header", name = "Bar sizing", order = 30, },
			
        width = {
			type = "range",
			name = "Width",
			desc = "Set the bar width",
			order = 40,
			cmdHidden = true,
			min = 100,
			max = 300,
			step = 1,
			get = function(info) return EnrageTracker.db.profile.width end,
			set = function(info, value) 
				EnrageTracker.db.profile.width = value
				EnrageTracker:SetupFrame()
				EnragedFrame:Start(10)
			end,
        },
		
        height = {
			type = "range",
			name = "Height",
			desc = "Set the bar height",
			order = 50,
			cmdHidden = true,
			min = 10,
			max = 50,
			step = 1,
			get = function(info) return EnrageTracker.db.profile.height end,
			set = function(info, value) 
				EnrageTracker.db.profile.height = value
				EnrageTracker:SetupFrame()
				EnragedFrame:Start(10)
			end,
        },
		
		scale = {
			type = "range",
			name = "Scale",
			desc = "Set the bar scale",
			order = 60,
			cmdHidden = true,
			min = 0.5,
			max = 2,
			step = 0.1,
			get = function(info) return EnrageTracker.db.profile.scale end,
			set = function(info, value) 
				EnrageTracker.db.profile.scale = value
				EnrageTracker:SetupFrame()
				EnragedFrame:Start(10)
			end,
        },
		
		barspacer = { type = "description", name = "\n", order = 64, },
		barheader = { type = "header", name = "Bar settings", order = 65, },
		
		color = {
			type = "color",
			name = "Bar Color",
			desc = "Set the bar color",
			order = 70,
			cmdHidden = true,
			hasAlpha = true,
			get = function(info) return unpack(EnrageTracker.db.profile.color) end,
			set = function(info, r, g, b, a)
				EnrageTracker.db.profile.color = { r, g, b, a }
				EnrageTracker:UpdateSettings()
				EnragedFrame:Start(10)
			end,
		},
		
		bgcolor = {
			type = "color",
			name = "Background Color",
			desc = "Set the background bar color",
			order = 75,
			cmdHidden = true,
			hasAlpha = true,
			get = function(info) return unpack(EnrageTracker.db.profile.bgcolor) end,
			set = function(info, r, g, b, a)
				EnrageTracker.db.profile.bgcolor = { r, g, b, a }
				EnrageTracker:UpdateSettings()
				EnragedFrame:Start(10)
			end,
		},
		
		texture = {
			type = "select",
			name = "Texture",
			desc = "Set the bar texture",
			order = 80,
			cmdHidden = true,
			values = SharedMedia:HashTable("statusbar"),
			dialogControl = "LSM30_Statusbar",
			get = function(info) return EnrageTracker.db.profile.texture end,
			set = function(info, value)
				EnrageTracker.db.profile.texture = value
				EnrageTracker:UpdateSettings()
				EnragedFrame:Start(10)
			end,
		},
	
		fontspacer = { type = "description", name = "\n", order = 89, },
		fontheader = { type = "header", name = "Text settings", order = 90, },
		
        font = {
			type = "select",
			name = "Font",
			desc = "Set the font",
			order = 100,
			cmdHidden = true,
			values = SharedMedia:HashTable("font"),
			dialogControl = "LSM30_Font",
			get = function(info) return EnrageTracker.db.profile.font end,
			set = function(info, value) 
				EnrageTracker.db.profile.font = value
				EnrageTracker:UpdateSettings()
				EnragedFrame:Start(10)
			end,
        },
		
		fontscale = {
			type = "range",
			name = "Scale",
			desc = "Set the font scale",
			order = 110,
			cmdHidden = true,
			min = 1,
			max = 50,
			step = 1,
			get = function(info) return EnrageTracker.db.profile.fontscale end,
			set = function(info, value) 
				EnrageTracker.db.profile.fontscale = value
				EnrageTracker:UpdateSettings()
				EnragedFrame:Start(10)
			end,
        },
		
		fontalignment = {
			type = "select",
			name = "Alignment",
			desc = "Text alignment",
			order = 115,
			cmdHidden = true,
			values = { ["LEFT"] = "Left", ["CENTER"] = "Center" },
			get = function(info) return EnrageTracker.db.profile.fontalignment end,
			set = function(info, value)
				EnrageTracker.db.profile.fontalignment = value
				EnrageTracker:UpdateSettings()
				EnragedFrame:Start(10)
			end,
        },
		
		buttonspacer = { type = "description", name = "\n", order = 119, },
		buttonheader = { type = "header", name = "Bar positioning", order = 120, },
		
		config = {
			type = "execute",
			name = "Config",
			desc = "Show the config panel",
			order = 130,
			guiHidden = true,
			func = "ShowConfig",
		},
		
		lock = { 
			type = "execute",
			name = "Lock",
			desc = "Lock the frame",
			order = 140,
			func = "LockFrame",
		},
		
		unlock = {
			type = "execute",
			name = "Unlock",
			desc = "Unlock the frame",
			order = 150,
			func = "UnlockFrame",
		},
		
		reset = {
			type = "execute",
			name = "Reset",
			desc = "Reset the frame",
			order = 160,
			func = "ResetFrame",
		},
		
		test = {
			type = "execute",
			name = "Test",
			desc = "Show a test bar",
			order = 170,
			func = function() EnragedFrame:Start(10) end,
		},
	},
}


-- Default profile settings
local defaults = {
	profile = {
		enable = true,
		fury = true,
		locked = false,

		width = 125,
		height = 15,
		scale = 1,
		
		point = "CENTER",
		xpos = 0,
		ypos = 0,

		color = { 1, 0, 0, 1 },
		bgcolor = { 0.5, 0.5, 0.5, 0.3 },
		texture = "Smooth",
		
        font = "Friz Quadrata TT",
        fontscale = 10,
		fontalignment = "CENTER",
	},
}


--------------------------------------------------------------------------------
-- Main functions
--------------------------------------------------------------------------------


-- OnInitialize() - Initialize the addon
function EnrageTracker:OnInitialize()
	-- Only load the addon for warriors
	if select(2, UnitClass("player")) ~= "WARRIOR" then
		EnrageTracker:Disable()
		return 
	end
	
	-- Load saved variables
	self.db = LibStub("AceDB-3.0"):New("EnrageTrackerDB", defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	-- Add the options table
	LibStub("AceConfig-3.0"):RegisterOptionsTable("EnrageTracker", options, {"enragetracker", "enrage"})
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("EnrageTracker", "EnrageTracker")
	
	-- Add the profiles table
	LibStub("AceConfig-3.0"):RegisterOptionsTable("EnragedProfiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db))
	self.optionsFrame.profiles = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("EnragedProfiles", "Profiles", "EnrageTracker")

	-- Initialize variables
	self.buffList = {}
	self.lastEnrage = 0

	-- Initialize the bar settings and the frame layout
	self:UpdateSettings()
	self:SetupFrame()
end


-- OnEnable() - Enable the addon
function EnrageTracker:OnEnable()
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED", "ToggleAddon")
	self:ToggleAddon()
end


-- OnDisable() - Disable the addon
function EnrageTracker:OnDisable()
	self:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	self:UnregisterEvent("UNIT_AURA")
end


-- ToggleAddon() - Enable or disable the addon
function EnrageTracker:ToggleAddon()
	if self.db.profile.enable and (not self.db.profile.fury or (self.db.profile.fury and GetPrimaryTalentTree() == 2)) then
		self:RegisterEvent("UNIT_AURA")
	else
		self:UnregisterEvent("UNIT_AURA")
	end
end


-- ProfileChanged() - Called whenever the profile is changed
function EnrageTracker:ProfileChanged(event, database)
	self.db = database
	self:UpdateSettings()
	self:SetupFrame()
	self:ToggleAddon()
end


-- ShowConfig() - Show the options frame
function EnrageTracker:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end


-- UNIT_AURA() - Check for enrage effects and display the bar
function EnrageTracker:UNIT_AURA(event, unit, ...)
	if unit ~= "player" then
		return
	end
		
	-- Clear the buff history if the bar isn't visible
	if not EnragedFrame:IsVisible() then
		self.buffList = {}
		self.lastEnrage = 0
	end

	-- Check for enrage effects
	local buff, expires
	for k,v in pairs(enrages) do
		buff, _, _, _, _, _, expires = UnitBuff("player", v)
		if buff then
			-- Add the buff
			if not self.buffList[buff] then
				self.buffList[buff] = 0
			end	
			
			-- Update the expiration date
			self.buffList[buff] = expires
		else
			-- Remove the buff
			if self.buffList[v] then
				self.buffList[v] = nil
			end
		end
	end

	-- Find the enrage effect with the longest remaining duration
	local name
	local longest = 0
	for k,v in pairs(self.buffList) do
		if v > longest then
			name = k
			longest = v
		end
	end

	-- Show the longest enrage effect
	if longest > self.lastEnrage then
		EnragedFrame:Start(longest - GetTime())
		self.lastEnrage = longest
	-- Hide the bar if there aren't any enrages
	elseif not name then
		EnragedFrame:Stop()
	end
end


-- UpdateSettings() - Set the bar texture, color, etc..
function EnrageTracker:UpdateSettings()
	EnragedFrame:SetTexture(SharedMedia:Fetch("statusbar", self.db.profile.texture))
	EnragedFrame:SetColor(self.db.profile.color, self.db.profile.bgcolor)
		
	EnragedFrame.nameLabel:SetFont(SharedMedia:Fetch("font", self.db.profile.font), self.db.profile.fontscale, nil)
    EnragedFrame.durationLabel:SetFont(SharedMedia:Fetch("font", self.db.profile.font), self.db.profile.fontscale, nil)
	EnragedFrame.nameLabel:SetJustifyH(self.db.profile.fontalignment)
end


--------------------------------------------------------------------------------
-- Frame functions
--------------------------------------------------------------------------------


--- SetupFrame() - Setup the enraged frame
function EnrageTracker:SetupFrame()
	EnragedFrame:SetSize(self.db.profile.width, self.db.profile.height)
	EnragedFrame:SetScale(self.db.profile.scale)
	
	EnragedFrame:ClearAllPoints()
	EnragedFrame:SetPoint(self.db.profile.point, nil, self.db.profile.point, self.db.profile.xpos, self.db.profile.ypos)
	
	if self.db.profile.locked then
		self:LockFrame()
	else
		self:UnlockFrame()
	end
end


-- LockFrame() - Lock the frame
function EnrageTracker:LockFrame()
	self.db.profile.locked = true
	EnragedFrame:Lock()
end


-- UnlockFrame() - Unlock the frame
function EnrageTracker:UnlockFrame()
	self.db.profile.locked = false
	EnragedFrame:Unlock()
end


-- ResetFrame() - Reset the frame
function EnrageTracker:ResetFrame()
	EnragedFrame:ClearAllPoints()
	EnragedFrame:SetPoint("CENTER", UIParent, "CENTER")
	self:UnlockFrame()
end


--- SavePosition() - Save the frame position and size
function EnrageTracker:SavePosition()
	self.db.profile.width, self.db.profile.height = EnragedFrame:GetSize()
	self.db.profile.point, _, _, self.db.profile.xpos, self.db.profile.ypos = EnragedFrame:GetPoint()
end
